@@ -4,7 +4,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <locale.h>
@@ -173,7 +172,8 @@ static time_t comparetime;
 static char *urlfile, **urls;
 static size_t nurls;
 
-volatile sig_atomic_t sigstate = 0;
+volatile sig_atomic_t state_sigchld = 0, state_sighup = 0, state_sigint = 0;
+volatile sig_atomic_t state_sigterm = 0, state_sigwinch = 0;
 
 static char *plumbercmd = "xdg-open"; /* env variable: $SFEED_PLUMBER */
 static char *pipercmd = "sfeed_content"; /* env variable: $SFEED_PIPER */
@@ -549,12 +549,13 @@ init(void)
 	cursormode(0);
 
 	if (usemouse)
-		mousemode(usemouse);
+		mousemode(1);
 
 	memset(&sa, 0, sizeof(sa));
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART; /* require BSD signal semantics */
 	sa.sa_handler = sighandler;
+	sigaction(SIGCHLD, &sa, NULL);
 	sigaction(SIGHUP, &sa, NULL);
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
@@ -564,25 +565,28 @@ init(void)
 void
 processexit(pid_t pid, int interactive)
 {
-	pid_t wpid;
 	struct sigaction sa;
 
 	memset(&sa, 0, sizeof(sa));
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART; /* require BSD signal semantics */
-	sa.sa_handler = SIG_IGN;
-	sigaction(SIGINT, &sa, NULL);
 
 	if (interactive) {
-		while ((wpid = wait(NULL)) >= 0 && wpid != pid)
-			;
+		/* ignore SIGINT (^C) in parent for interactive applications */
+		sa.sa_handler = SIG_IGN;
+		sigaction(SIGINT, &sa, NULL);
+
+		sa.sa_flags = 0; /* SIGTERM: interrupt waitpid(), no SA_RESTART */
+		sa.sa_handler = sighandler;
+		sigaction(SIGTERM, &sa, NULL);
+
+		/* wait for process to change state, ignore errors */
+		waitpid(pid, NULL, 0);
+
 		init();
 		updatesidebar();
 		updategeom();
 		updatetitle();
-	} else {
-		sa.sa_handler = sighandler;
-		sigaction(SIGINT, &sa, NULL);
 	}
 }
 
@@ -606,8 +610,8 @@ pipeitem(const char *cmd, struct item *item, int field, int interactive)
 		die("fork");
 	case 0:
 		if (!interactive) {
-			dup2(devnullfd, 1);
-			dup2(devnullfd, 2);
+			dup2(devnullfd, 1); /* stdout */
+			dup2(devnullfd, 2); /* stderr */
 		}
 
 		errno = 0;
@@ -644,8 +648,9 @@ forkexec(char *argv[], int interactive)
 		die("fork");
 	case 0:
 		if (!interactive) {
-			dup2(devnullfd, 1);
-			dup2(devnullfd, 2);
+			dup2(devnullfd, 0); /* stdin */
+			dup2(devnullfd, 1); /* stdout */
+			dup2(devnullfd, 2); /* stderr */
 		}
 		if (execvp(argv[0], argv) == -1)
 			_exit(1);
@@ -985,8 +990,10 @@ lineeditor(void)
 	size_t cap = 0, nchars = 0;
 	int ch;
 
+	if (usemouse)
+		mousemode(0);
 	for (;;) {
-		if (nchars + 1 >= cap) {
+		if (nchars + 2 >= cap) {
 			cap = cap ? cap * 2 : 32;
 			input = erealloc(input, cap);
 		}
@@ -999,30 +1006,33 @@ lineeditor(void)
 			if (!nchars)
 				continue;
 			input[--nchars] = '\0';
-			write(1, "\b \b", 3); /* back, blank, back */
-			continue;
+			ttywrite("\b \b"); /* back, blank, back */
 		} else if (ch >= ' ') {
 			input[nchars] = ch;
-			write(1, &input[nchars], 1);
+			input[nchars + 1] = '\0';
+			ttywrite(&input[nchars]);
 			nchars++;
 		} else if (ch < 0) {
-			switch (sigstate) {
-			case 0:
-			case SIGWINCH:
-				/* continue editing: process signal later */
-				continue;
-			case SIGINT:
-				/* cancel prompt, but do not quit */
-				sigstate = 0; /* reset: do not handle it */
-				break;
-			default: /* other: SIGHUP, SIGTERM */
-				/* cancel prompt and handle signal after */
-				break;
+			if (state_sigchld) {
+				state_sigchld = 0;
+				/* wait on child processes so they don't become a zombie */
+				while (waitpid((pid_t)-1, NULL, WNOHANG) > 0)
+					;
 			}
+			if (state_sigint)
+				state_sigint = 0; /* cancel prompt and don't handle this signal */
+			else if (state_sighup || state_sigterm)
+				; /* cancel prompt and handle these signals */
+			else /* no signal, time-out or SIGCHLD or SIGWINCH */
+				continue; /* do not cancel: process signal later */
+
 			free(input);
-			return NULL;
+			input = NULL;
+			break; /* cancel prompt */
 		}
 	}
+	if (usemouse)
+		mousemode(1);
 	return input;
 }
 
@@ -1308,7 +1318,8 @@ feeds_load(struct feed *feeds, size_t nfeeds)
 	struct feed *f;
 	size_t i;
 
-	if ((comparetime = time(NULL)) == -1)
+	errno = 0;
+	if ((comparetime = time(NULL)) == (time_t)-1)
 		die("time");
 	/* 1 day is old news */
 	comparetime -= 86400;
@@ -1579,14 +1590,11 @@ void
 sighandler(int signo)
 {
 	switch (signo) {
-	case SIGHUP:
-	case SIGINT:
-	case SIGTERM:
-	case SIGWINCH:
-		/* SIGTERM is more important, do not override it */
-		if (sigstate != SIGTERM)
-			sigstate = signo;
-		break;
+	case SIGCHLD:  state_sigchld = 1;  break;
+	case SIGHUP:   state_sighup = 1;   break;
+	case SIGINT:   state_sigint = 1;   break;
+	case SIGTERM:  state_sigterm = 1;  break;
+	case SIGWINCH: state_sigwinch = 1; break;
 	}
 }
 
@@ -1835,7 +1843,7 @@ markread(struct pane *p, off_t from, off_t to, int isread)
 	FILE *fp;
 	off_t i;
 	const char *cmd;
-	int isnew = !isread, pid, wpid, status, visstart;
+	int isnew = !isread, pid, status = -1, visstart;
 
 	if (!urlfile || !p->nrows)
 		return;
@@ -1846,8 +1854,8 @@ markread(struct pane *p, off_t from, off_t to, int isread)
 	case -1:
 		die("fork");
 	case 0:
-		dup2(devnullfd, 1);
-		dup2(devnullfd, 2);
+		dup2(devnullfd, 1); /* stdout */
+		dup2(devnullfd, 2); /* stderr */
 
 		errno = 0;
 		if (!(fp = popen(cmd, "w")))
@@ -1866,11 +1874,9 @@ markread(struct pane *p, off_t from, off_t to, int isread)
 		status = WIFEXITED(status) ? WEXITSTATUS(status) : 127;
 		_exit(status);
 	default:
-		while ((wpid = wait(&status)) >= 0 && wpid != pid)
-			;
-
-		/* fail: exit statuscode was non-zero */
-		if (status)
+		/* waitpid() and block on process status change,
+		   fail if exit statuscode was unavailable or non-zero */
+		if (waitpid(pid, &status, 0) <= 0 || status)
 			break;
 
 		visstart = p->pos - (p->pos % p->height); /* visible start */
@@ -2113,24 +2119,26 @@ main(int argc, char *argv[])
 			case 'D': goto keyleft;  /* arrow left */
 			case 'F': goto endpos;   /* end */
 			case 'H': goto startpos; /* home */
-			case '1': /* home */
-			case '4': /* end */
-			case '5': /* page up */
-			case '6': /* page down */
-			case '7': /* home: urxvt */
-			case '8': /* end: urxvt */
-				i = ch;
-				if ((ch = readch()) < 0)
-					goto event;
-				if (ch == '~') {
-					switch (i) {
-					case '1': goto startpos;
-					case '4': goto endpos;
-					case '5': goto prevpage;
-					case '6': goto nextpage;
-					case '7': goto startpos;
-					case '8': goto endpos;
+			default:
+				if (!(ch >= '0' && ch <= '9'))
+					break;
+				for (i = ch - '0'; ;) {
+					if ((ch = readch()) < 0) {
+						goto event;
+					} else if (ch >= '0' && ch <= '9') {
+						i = (i * 10) + (ch - '0');
+						continue;
+					} else if (ch == '~') { /* DEC: ESC [ num ~ */
+						switch (i) {
+						case 1: goto startpos; /* home */
+						case 4: goto endpos;   /* end */
+						case 5: goto prevpage; /* page up */
+						case 6: goto nextpage; /* page down */
+						case 7: goto startpos; /* home: urxvt */
+						case 8: goto endpos;   /* end: urxvt */
+						}
 					}
+					break;
 				}
 			}
 			break;
@@ -2334,23 +2342,35 @@ nextpage:
 event:
 		if (ch == EOF)
 			goto end;
-		else if (ch == -3 && sigstate == 0)
+		else if (ch == -3 && !state_sigchld && !state_sighup &&
+		         !state_sigint && !state_sigterm && !state_sigwinch)
 			continue; /* just a time-out, nothing to do */
 
-		switch (sigstate) {
-		case SIGHUP:
-			feeds_reloadall();
-			sigstate = 0;
-			break;
-		case SIGINT:
-		case SIGTERM:
+		/* handle signals in a particular order */
+		if (state_sigchld) {
+			state_sigchld = 0;
+			/* wait on child processes so they don't become a zombie,
+			   do not block the parent process if there is no status,
+			   ignore errors */
+			while (waitpid((pid_t)-1, NULL, WNOHANG) > 0)
+				;
+		}
+		if (state_sigterm) {
 			cleanup();
-			_exit(128 + sigstate);
-		case SIGWINCH:
+			_exit(128 + SIGTERM);
+		}
+		if (state_sigint) {
+			cleanup();
+			_exit(128 + SIGINT);
+		}
+		if (state_sighup) {
+			state_sighup = 0;
+			feeds_reloadall();
+		}
+		if (state_sigwinch) {
+			state_sigwinch = 0;
 			resizewin();
 			updategeom();
-			sigstate = 0;
-			break;
 		}
 
 		draw();
