@@ -63,7 +63,8 @@ printfeed(FILE *fp, const char *feedname)
 	ssize_t linelen;
 	int ishtml;
 
-	while ((linelen = getline(&line, &linesize, fp)) > 0) {
+	while ((linelen = getline(&line, &linesize, fp)) > 0 &&
+	       !ferror(stdout)) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
 		hash = djb2((unsigned char *)line, 5381ULL);
@@ -152,8 +153,8 @@ main(int argc, char *argv[])
 		user = "you";
 	if (gethostname(host, sizeof(host)) == -1)
 		err(1, "gethostname");
-	if ((now = time(NULL)) == -1)
-		err(1, "time");
+	if ((now = time(NULL)) == (time_t)-1)
+		errx(1, "time");
 	if (!gmtime_r(&now, &tmnow))
 		err(1, "gmtime_r: can't get current time");
 	if (!strftime(mtimebuf, sizeof(mtimebuf), "%a %b %d %H:%M:%S %Y", &tmnow))
@@ -163,17 +164,20 @@ main(int argc, char *argv[])
 
 	if (argc == 1) {
 		printfeed(stdin, "");
+		checkfileerror(stdin, "<stdin>", 'r');
 	} else {
 		for (i = 1; i < argc; i++) {
 			if (!(fp = fopen(argv[i], "r")))
 				err(1, "fopen: %s", argv[i]);
 			name = ((name = strrchr(argv[i], '/'))) ? name + 1 : argv[i];
 			printfeed(fp, name);
-			if (ferror(fp))
-				err(1, "ferror: %s", argv[i]);
+			checkfileerror(fp, argv[i], 'r');
+			checkfileerror(stdout, "<stdout>", 'w');
 			fclose(fp);
 		}
 	}
 
+	checkfileerror(stdout, "<stdout>", 'w');
+
 	return 0;
 }
