@@ -1,6 +1,5 @@
-#include <sys/types.h>
-
 #include <stdio.h>
+#include <time.h>
 
 #ifdef __OpenBSD__
 #include <unistd.h>
@@ -9,6 +8,13 @@
 #define unveil(p1,p2) 0
 #endif
 
+/* ctype-like macros, but always compatible with ASCII / UTF-8 */
+#define ISALPHA(c) ((((unsigned)c) | 32) - 'a' < 26)
+#define ISCNTRL(c) ((c) < ' ' || (c) == 0x7f)
+#define ISDIGIT(c) (((unsigned)c) - '0' < 10)
+#define ISSPACE(c) ((c) == ' ' || ((((unsigned)c) - '\t') < 5))
+#define TOLOWER(c) ((((unsigned)c) - 'A' < 26) ? ((c) | 32) : (c))
+
 #undef strcasestr
 char *strcasestr(const char *, const char *);
 #undef strlcat
@@ -64,6 +70,7 @@ int uri_hasscheme(const char *);
 int uri_makeabs(struct uri *, struct uri *, struct uri *);
 int uri_parse(const char *, struct uri *);
 
+void checkfileerror(FILE *, const char *, int);
 void parseline(char *, char *[FieldLast]);
 void printutf8pad(FILE *, const char *, size_t, int);
 int  strtotime(const char *, time_t *);
